use serde::{Deserialize, Serialize};
use serde::ser::{Serializer, SerializeMap};


type JobID = String;
type NodeId = i32;

#[derive(Serialize, Deserialize)]
pub struct JobLog {
  pub succeeded : i32,
  pub failed    : i32,
  pub remaining : i32,
  // , events    : !(Maybe [ScraperEvent])
}

pub enum CEMessage {
    UpdateJobProgress { j_id: JobID, job_log: JobLog },
    UpdateTreeFirstLevel { node_id: NodeId },
}
impl Serialize for CEMessage {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
	S: Serializer
    {
	let mut ret = serializer.serialize_map(None)?;

	match self {
	    CEMessage::UpdateJobProgress { j_id, job_log } => {
		ret.serialize_entry("type", "update_job_progress")?;
		ret.serialize_entry("j_id", j_id)?;
		ret.serialize_entry("job_log", job_log)?;
	    },
	    CEMessage::UpdateTreeFirstLevel { node_id } => {
		ret.serialize_entry("type", "update_tree_first_level")?;
		ret.serialize_entry("node_id", node_id)?;
	    },
	}
	
	ret.end()
    }
}
